#ifndef GAME_IMPL_H
#define GAME_IMPL_H

#include <Game_skel.h>
#include <Game.h>

// STL
#include <vector>
#include <hash_map>

#define FAILED -2

struct bomb_compare {
  bool operator()(int x, int y) const {
    return (x > y);
  }
};

class Game_impl : public POA_Game,
                  public PortableServer::RefCountServantBase
{
  private:
    // The ORB
    CORBA::ORB_var orb_;

    // The servants default POA
    PortableServer::POA_var poa_;

    // initialize the game state each round
    void init_round();

    // get the PlayerInfo struct that corresponds to the give player ID
    struct PlayerInfo* get_player(unsigned int pid);

    // game ID for this game
    int game_id;

    // current game state to send back to clients
    struct GameState gs;

    // Check to see if somebody has won the game.
    int check_for_winner();

    // Resolves the effects of bombs set to explode
    void resolve_fire();
    // Helper function for resolve_fire();
    void resolve_fire_radius( int radius, int direction );

    hash_map<int, struct BombInfo *, hash<int>, bomb_compare> bomb_loc;
    hash_map<int, struct BombInfo *, hash<int>, bomb_compare>::iterator bomb_loc_iter;
    // List of bombs set to explode
    vector<struct BombInfo *> exploders;
    vector<struct BombInfo *>::iterator exploders_iter;

  public:

    Game_impl(CORBA::ORB_ptr orb, PortableServer::POA_ptr poa, 
        CORBA::ULong game_id);

    ~Game_impl();

    struct GameState Game_impl::update(const struct ClientAction &c_action)
      throw (CORBA::SystemException);

    long Game_impl::exit_game(CORBA::ULong player_id, CORBA::ULong game_id)
      throw (CORBA::SystemException);

    bool Game_impl::is_game_started(CORBA::ULong player_id, 
        CORBA::ULong game_id)
      throw (CORBA::SystemException);

    long Game_impl::get_id()
      throw (CORBA::SystemException);

    // Insert a new player to the game by taking a unique player ID
    long insert_player(CORBA::ULong player_id)
      throw(CORBA::SystemException);

    virtual PortableServer::POA_ptr _default_POA();
};
#endif

