#ifndef __SQL_LIB_H__
#define __SQL_LIB_H__

#include <OB/CORBA.h>
#include <Game.h>


int db_connect();
int db_disconnect();

// Creates the game tables if they do not already exist in the database
void db_create_tables();

int db_drop_all_tables();

int db_populate_tables();

// Returns game id of latest waiting game or -1 if it is active
int db_check_games();

// Returns id of newly-created game
int db_create_game();

// Returns id of newly-created player
int db_create_player(int game_id, int x_coord, int y_coord);

// Returns DELETE_GAME (-2) if there are no more players left or SUCCESS (0) otherwise or ERROR (-1)
int db_exit_game(int game_id, int player_id);

// Stores entire gamestate into gs
int db_get_gamestate(int game_id, struct GameState *gs);

// Stores PlayerInfo into gs->players[i]
int db_get_all_players(struct GameState *gs);

// Stores BombInfo into pi->bombs[i]
int db_get_all_bombs(struct PlayerInfo *pi);

// Stores Map into gs->map_info
int db_get_map(struct GameState *gs);

// Returns 0 if player created onto existing game or -1 if added to new game
int db_add_player(int *game_id, int *player_id);

int db_move_player(int game_id, int player_id, int x_coord, int y_coord);

// FIX: DOES NOT STORE TIMELEFT YET
int db_plant_bomb(int game_id, int player_id, int x_coord, int y_coord);

// For atomic transactions:
// BEGIN will start a transaction
// COMMIT will commit all queries since BEGIN
// ROLLBACK will end transaction; all queries since BEGIN are lost
int db_begin_transaction();
int db_commit_transaction();
int db_rollback_transaction();

int db_lock_table(char *table);
int db_unlock_all_tables();


// convert the text version of the map to/from the array version in gs
void map_str_to_array(const char *map_str, struct GameState *gs);
void map_array_to_str(char *map_str, const struct GameState *gs);  // caller must allocate the map_str buffer


#endif
