/************
 * INCLUDES *
 ************/

// CORBA includes
#include <OB/CORBA.h>

// project includes
#include <GameServer_impl.h>
#include <Game_impl.h>
#include "sql_lib.h"


#ifdef HAVE_STD_IOSTREAM
using namespace std;
#endif

// ----------------------------------------------------------------------
// GameServer_impl constructor
// ----------------------------------------------------------------------

GameServer_impl::GameServer_impl(CORBA::ORB_ptr orb, 
                                 PortableServer::POA_ptr poa)
  : orb_(CORBA::ORB::_duplicate(orb)), 
  poa_(PortableServer::POA::_duplicate(poa))

{
  // Put Constructor here! 
}

// ----------------------------------------------------------------------
// GameServer_impl Destructor
// ----------------------------------------------------------------------

GameServer_impl::~GameServer_impl() {
  // Destructor here!
}

Game_ptr 
GameServer_impl::join_game(CORBA::ULong_out player_id, CORBA::ULong_out game_id)
  throw(CORBA::SystemException)
{
  // add_player returns 0 if player created onto existing game or -1 if added to new game
  int flag = db_add_player(&(int)game_id, &(int)player_id);
  printf("Adding player with PlayerId=%d\n",(int)player_id);

  // Create gameId object based on game_id returned from DB
  char gameIdStr[50];
  sprintf(gameIdStr,"%d",(int)game_id);
  PortableServer::ObjectId_var gameId = PortableServer::string_to_ObjectId(gameIdStr);

  // Get the default POA to associate this object to
  PortableServer::POA_var poa = _default_POA();

  if(flag == -1)
    {
      printf("Creating new game with gameId=%d\n",(int)game_id);
      // Create the game object
      Game_impl* gameImpl = new Game_impl(orb_,poa_,game_id);
      //PortableServer::ServantBase_var game_servant = gameImpl;
  
      // activate the game to the POA
      poa -> activate_object_with_id(gameId, gameImpl); 
    }

  // Get reference to the game object
  CORBA::Object_var gameObj = poa -> id_to_reference(gameId);
  Game_var game = Game::_narrow(gameObj);

  // register the playerID to the game object
  game -> insert_player((CORBA::ULong)player_id);

  // return a reference to the game object
  return Game::_narrow(poa -> id_to_reference(gameId));
}

PortableServer::POA_ptr
GameServer_impl::_default_POA()
{
  return PortableServer::POA::_duplicate(poa_);
}




