#include <mysql.h> /* Headers for MySQL usage */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "sql_lib.h"
#include "game_constants.h"


/***********
 * GLOBALS *
 ***********/

static MYSQL borba_db;
static char *host = "mysql.ece.cmu.edu";
static char *username = "c18749t7";
static char *password = "borba";


/*************
 * FUNCTIONS *
 *************/

/*
void main()
{
  struct GameState gamestate;
  struct GameState *gs = &gamestate;

  db_connect();
  db_drop_all_tables();
  db_create_tables();
  db_populate_tables();

  // game_id, gamestate
  db_get_gamestate(1, &gamestate);

  cout << "OK\n";

  // test it!
  cout << "GameState: "
       << "\ngame_id = " << gs->game_id
       << "\nseq_num = " << gs->seq_num
       << "\nstatus = " << gs->status
       << "\nnumber_players = " << gs->number_players
       << "\nround number = " << gs->round_number
       << "\ntime_left = " << gs->time_left << endl;

  int i;
  for(i = 0; i < gs->number_players; i++){
    struct PlayerInfo *pi = &(gs->players[i]);

    cout << "PlayerInfo: "
	 << "\nplayer_id = " << pi->player_id
	 << "\ngame_id = " << pi->game_id
	 << "\nstatus = " << pi->status
	 << "\nx_coord = " << pi->x_coord
	 << "\ny_coord = " << pi->y_coord
	 << "\nbombs_dropped = " << pi->bombs_dropped
	 << "\nrounds_won = " << pi->rounds_won << endl;

    int j;
    for(j = 0; j < pi->bombs_dropped; j++){
      struct BombInfo *bombs = &(pi->bombs[j]);

      cout << "BombInfo: "
	   << "\nplayer_id = " << bombs->player_id
	   << "\nx_coord = " << bombs->x_coord
	   << "\ny_coord = " << bombs->y_coord << endl;
    }
  }

  return 0;
}
*/

int db_connect()
{
  /* Make connection with mysql_connect(MYSQL db, char *host, char *username,
     char *password */
  if(!mysql_connect(&borba_db, host, username, password)){ 
    printf(mysql_error(&borba_db));
    return ERROR;
  }

  /* Select the database we want to use */
  if(mysql_select_db(&borba_db, username) != SUCCESS){
    printf(mysql_error(&borba_db));
    return ERROR;
  }

  return 0;
}

int db_disconnect()
{
  mysql_close(&borba_db);
  return SUCCESS;
}

void db_create_tables()
{
  char query[255];

  // Create GameState table
  sprintf(query,
	  "CREATE TABLE GameState ("
	  "id int NOT NULL AUTO_INCREMENT PRIMARY KEY, "
	  "seq_num int, "
	  "status int, "
	  "round_number int, "
	  "time_left int)");
  mysql_real_query(&borba_db, query, strlen(query));

  // Create Player table
  sprintf(query, 
	  "CREATE TABLE Player ("
	  "id int NOT NULL AUTO_INCREMENT PRIMARY KEY, "
	  "game_id int, "
	  "player_number int, "
	  "status int, "
	  "x_coord int, "
	  "y_coord int, "
	  "bombs_max int, "
	  "rounds_won int, "
	  "FOREIGN KEY (game_id) REFERENCES GameState(id))");
  mysql_real_query(&borba_db, query, strlen(query));

  // Create Bomb table
  sprintf(query,
	  "CREATE TABLE Bomb ("
	  "id int NOT NULL AUTO_INCREMENT PRIMARY KEY, "
	  "player_id int, "
	  "x_coord int, "
	  "y_coord int, "
	  "explode_time int, "
	  "FOREIGN KEY (player_id) REFERENCES Player(id))");
  mysql_real_query(&borba_db, query, strlen(query));

  // Create Map table
  sprintf(query,
	  "CREATE TABLE Map ("
	  "game_id int, "
	  "mapstring VARCHAR(%d), "
	  "FOREIGN KEY (game_id) REFERENCES GameState(id))",
	  (NUM_ROWS*NUM_COLS+1));
  mysql_real_query(&borba_db, query, strlen(query));
}

int db_drop_all_tables()
{
  char query[255];

  sprintf(query, "DROP TABLE GameState");
  mysql_real_query(&borba_db, query, strlen(query));

  sprintf(query, "DROP TABLE Bomb");
  mysql_real_query(&borba_db, query, strlen(query));

  sprintf(query, "DROP TABLE Player");
  mysql_real_query(&borba_db, query, strlen(query));

  sprintf(query, "DROP TABLE Map");
  mysql_real_query(&borba_db, query, strlen(query));

  return SUCCESS;
}

int db_populate_tables()
{
  char query[255];

  // GameState
  sprintf(query,
	  "INSERT INTO GameState (seq_num, status, round_number, time_left) VALUES (0,1,2,3)");
  mysql_real_query(&borba_db, query, strlen(query));

  sprintf(query,
	  "INSERT INTO GameState (seq_num, status, round_number, time_left) VALUES (110,111,112,113)");
  mysql_real_query(&borba_db, query, strlen(query));

  // Player
  sprintf(query,
	  "INSERT INTO Player (game_id, player_number, status, x_coord, y_coord, bombs_max, rounds_won) VALUES (1,1,1,0,0,3,1)");
  mysql_real_query(&borba_db, query, strlen(query));

  sprintf(query,
	  "INSERT INTO Player (game_id, player_number, status, x_coord, y_coord, bombs_max, rounds_won) VALUES (1,2,1,0,14,3,2)");
  mysql_real_query(&borba_db, query, strlen(query));

  // Bomb
  sprintf(query,
	  "INSERT INTO Bomb (player_id, x_coord, y_coord, explode_time) VALUES (1,0,0,50)");
  mysql_real_query(&borba_db, query, strlen(query));

  sprintf(query,
	  "INSERT INTO Bomb (player_id, x_coord, y_coord, explode_time) VALUES (1,3,4,70)");
  mysql_real_query(&borba_db, query, strlen(query));

}

int db_check_games()
{
  char query[255];
  int  game_id;
  MYSQL_RES *res;
  MYSQL_ROW row;

  // Select the latest game (has the largest id)
  sprintf(query,
	  "SELECT id, status FROM GameState "
	  "ORDER BY id DESC "
	  "LIMIT 1");
  printf("query: %s\n", query);

  if(mysql_real_query(&borba_db, query, strlen(query)) != 0){
    printf(mysql_error(&borba_db));
    return ERROR;
  }

  res = mysql_store_result(&borba_db);
  row = mysql_fetch_row(res);

  // No games have been created yet
  if(row == NULL)
    game_id = -1;
  else {
    // Count number of players in this game
    game_id = atoi(row[0]);
    int status = atoi(row[1]);
    mysql_free_result(res);

    sprintf(query, "SELECT COUNT(DISTINCT id) FROM Player "
	    "WHERE game_id = %d", game_id);

    if(mysql_real_query(&borba_db, query, strlen(query)) != 0){
      printf(mysql_error(&borba_db));
      return ERROR;
    }

    res = mysql_store_result(&borba_db);
    row = mysql_fetch_row(res);

    int number_players = atoi(row[0]);
    if(status != GSTAT_WAITING || number_players >= NUM_PLAYERS)
      game_id = -1;
  }

  mysql_free_result(res);
  //printf("Game_id = %d\n", game_id);

  return game_id;
}

int db_create_game()
{
  char query[255];
  int  insert_id;
  
  // FIX: DUMMY VALUES
  sprintf(query,
	  "INSERT INTO GameState "
	  "(seq_num, status, round_number, time_left) "
	  "VALUES (0, '%d', 0, 0)", GSTAT_WAITING);
  printf("query: %s\n", query);

  if(mysql_real_query(&borba_db, query, strlen(query)) != 0){
    printf(mysql_error(&borba_db));
    return ERROR;
  }

  // Find what id the new game was given
  insert_id = mysql_insert_id(&borba_db);

  return insert_id;
}

int db_get_gamestate(int game_id, struct GameState *gs)
{
  char query[255];
  MYSQL_RES *res;
  MYSQL_ROW row;

  sprintf(query,
	  "SELECT * FROM GameState WHERE id = %d", game_id);

  if(mysql_real_query(&borba_db, query, strlen(query)) != 0){
    printf(mysql_error(&borba_db));
    return ERROR;
  }

  res = mysql_store_result(&borba_db);
  row = mysql_fetch_row(res);

  // Game not found
  if(row == NULL)
    return ERROR;

  gs->game_id = strtoul(row[0], NULL, 10);
  gs->seq_num = strtoul(row[1], NULL, 10);
  gs->status  = strtoul(row[2], NULL, 10);
  gs->round_number = strtoul(row[3], NULL, 10);
  gs->time_left = strtoul(row[4], NULL, 10);

  //mysql_free_result(res);

  sprintf(query, "SELECT COUNT(DISTINCT id) FROM Player WHERE game_id = %d",
	  gs->game_id);

  //res = mysql_store_result(&borba_db);
  //row = mysql_fetch_row(res);

  if(row == NULL)
    gs->number_players = 0;
  else
    gs->number_players = strtoul(row[0], NULL, 10);

  mysql_free_result(res);

  // Fills up the players struct
  if(db_get_all_players(gs) == ERROR)
    return ERROR;

  // Fills up the map array
  if(db_get_map(gs) == ERROR)
    return ERROR;

  return SUCCESS;
}

int db_create_player(int game_id, int x_coord, int y_coord)
{
  char query[255];
  int  insert_id;
  MYSQL_RES *res;
  MYSQL_ROW row;
  bool pnums[NUM_PLAYERS];  // stores TRUE for player numbers that are in use
  int player_num;
  for (int i = 0; i < NUM_PLAYERS; i++) {
    pnums[i] = false;
  }

  db_begin_transaction();

  // figure out which player number to use
  sprintf(query, 
          "SELECT player_number FROM Player WHERE game_id = %d",
          game_id);
  if (mysql_real_query(&borba_db, query, strlen(query)) != 0) {
    db_rollback_transaction();
    printf(mysql_error(&borba_db));
    return ERROR;
  }
  res = mysql_store_result(&borba_db);
  while ((row = mysql_fetch_row(res)) != NULL) {
    // NOTE: player_number values go from 1-4 instead of 0-3
    player_num = strtoul(row[0], NULL, 10);
    if ((player_num >= 1) && (player_num <= NUM_PLAYERS)) {
      pnums[player_num-1] = true;
    } else {
      cerr << "ERROR: database has invalid player_number value: " 
           << player_num << endl;
    }
  }

  // find the first number not in use
  player_num = 0;
  for (int i = 0; i < NUM_PLAYERS; i++) {
    if (pnums[i] == false) {
      player_num = i+1;
    }
  }
  

  // Create a new player
  sprintf(query, 
	  "INSERT INTO Player "
	  "(game_id, player_number, status, x_coord, y_coord, "
	  "bombs_max, rounds_won) "
	  "VALUES ('%d', '%d', '%d', '%d', '%d', '%d', '%d')",
	  game_id, player_num, PSTAT_ALIVE, x_coord, y_coord, MAX_BOMBS, 0);
  printf("query: %s\n", query);

  if(mysql_real_query(&borba_db, query, strlen(query)) != 0){
    printf(mysql_error(&borba_db));
    return ERROR;
  }

  // Find what id the new player was given
  insert_id = mysql_insert_id(&borba_db);

  return insert_id;
}

int db_exit_game(int game_id, int player_id)
{
  char query[255];
  MYSQL_RES *res;
  MYSQL_ROW row;

  //db_begin_transaction();

  sprintf(query, "UPDATE Player SET status = %d", PSTAT_LEFTGAME);
  if(mysql_real_query(&borba_db, query, strlen(query)) != 0){
    db_rollback_transaction();
    printf(mysql_error(&borba_db));
    return ERROR;
  }

  // FIXME: If this is the last player, do clean up
  /*
    // Delete all players from Player
    sprintf(query,
    "DELETE FROM Player WHERE id = %d", player_id);
    printf("query: %s\n", query);

    if(mysql_real_query(&borba_db, query, strlen(query)) != 0){
    db_rollback_transaction();
    printf(mysql_error(&borba_db));
    return ERROR;
    }

    db_commit_transaction();
  */

  // FIXME: is checking status != PSTAT_LEFTGAME foolproof?
  sprintf(query,
	  "SELECT COUNT(DISTINCT id) FROM GameState "
	  "WHERE id = %d AND status != %d",
	  game_id, PSTAT_LEFTGAME);

  res = mysql_store_result(&borba_db);
  row = mysql_fetch_row(res);

  if(row == NULL)
    return ERROR;
  else {
    if(atoi(row[0]) == 0) {
      mysql_free_result(res);
      return DELETE_GAME;
    }
  }

  return SUCCESS;
}

int db_add_player(int *game_id, int *player_id)
{
  int create_game_flag = 0;

  // Create a new game if all games are full
  *game_id = db_check_games();
  if(*game_id == -1){
    *game_id = db_create_game();
    create_game_flag = -1;
  }

  // FIX: SET CORRECT COORDINATES
  *player_id = db_create_player(*game_id, 0, 0);

  return create_game_flag;
}

int db_get_all_players(struct GameState *gs)
{
  char query[255];
  MYSQL_RES *res;
  MYSQL_ROW row;

  sprintf(query,
	  "SELECT * FROM Player WHERE game_id = %d ORDER BY player_number ASC",
	  gs->game_id);

  if(mysql_real_query(&borba_db, query, strlen(query)) != 0){
    printf(mysql_error(&borba_db));
    return ERROR;
  }

  res = mysql_store_result(&borba_db);

  int count = 0;
  while((row = mysql_fetch_row(res)) != NULL){
    struct PlayerInfo *pi = &(gs->players[count]);

    pi->player_id = strtoul(row[0], NULL, 10);
    pi->game_id   = strtoul(row[1], NULL, 10);
    // No need to store player_number
    pi->status    = strtoul(row[3], NULL, 10);  
    pi->x_coord   = strtoul(row[4], NULL, 10);
    pi->y_coord   = strtoul(row[5], NULL, 10);
    pi->rounds_won = strtoul(row[7], NULL, 10);

    int bombs_dropped;
    if((bombs_dropped = db_get_all_bombs(pi)) == ERROR){
      mysql_free_result(res);
      return ERROR;
    }
    else
      pi->bombs_dropped = (unsigned long)bombs_dropped;

    ++count;
  }

  mysql_free_result(res);

  return SUCCESS;
}

int db_get_all_bombs(struct PlayerInfo *pi)
{
  char query[255];
  MYSQL_RES *res;
  MYSQL_ROW row;

  sprintf(query,
	  "SELECT * FROM Bomb WHERE player_id = %d",
	  pi->player_id);
    
  if(mysql_real_query(&borba_db, query, strlen(query)) != 0){
    printf(mysql_error(&borba_db));
    return ERROR;
  }

  res = mysql_store_result(&borba_db);

  int count = 0;
  while((row = mysql_fetch_row(res)) != NULL){
    struct BombInfo *bomb = &(pi->bombs[count]);
    
    bomb->player_id = strtoul(row[1], NULL, 10);
    bomb->x_coord   = strtoul(row[2], NULL, 10);
    bomb->y_coord   = strtoul(row[3], NULL, 10);

    ++count;
  }

  mysql_free_result(res);

  return count;
}

int db_get_map(struct GameState *gs)
{
  char query[255];
  MYSQL_RES *res;
  MYSQL_ROW row;

  sprintf(query, "SELECT mapstring FROM Map WHERE game_id = %d",
	  gs->game_id);

  if(mysql_real_query(&borba_db, query, strlen(query)) != 0){
    printf(mysql_error(&borba_db));
    return ERROR;
  }

  res = mysql_store_result(&borba_db);
  row = mysql_fetch_row(res);

  if(row == NULL)
    return ERROR;
  else {
    map_str_to_array(row[0], gs);
  }

  mysql_free_result(res);

  return SUCCESS;
}

int db_move_player(int game_id, int player_id, int x_coord, int y_coord)
{
  char query[255];

  sprintf(query,
	  "UPDATE Player "
	  "SET game_id = %d, x_coord = %d, y_coord = %d "
	  "WHERE id = %d", 
	  game_id, x_coord, y_coord, player_id);

  if(mysql_real_query(&borba_db, query, strlen(query)) != 0){
    printf(mysql_error(&borba_db));
    return ERROR;
  }

  return SUCCESS;
}

int db_plant_bomb(int game_id, int player_id, int x_coord, int y_coord)
{
  char query[255];

  db_begin_transaction(); 

  // FIX: NEED TO GET TIME FROM MYSQL AND STORE INTO BOMB
  sprintf(query,
	  "INSERT INTO Bomb (player_id, x_coord, y_coord, explode_time) "
	  "VALUES (%d, %d, %d, 0)",
	  player_id, x_coord, y_coord);

  if(mysql_real_query(&borba_db, query, strlen(query)) != 0){
    db_rollback_transaction();
    printf(mysql_error(&borba_db));
    return ERROR;
  }

  /*
    sprintf(query,
    "UPDATE Player SET bombs_dropped = bombs_dropped + 1 "
    "WHERE player_id = %d",
    player_id);

    if(mysql_real_query(&borba_db, query, strlen(query)) != 0){
    db_rollback_transaction();
    printf(mysql_error(&borba_db));
    return ERROR;
    }
  */
  db_commit_transaction();

  // DEBUG
  // Find what id the new bomb was given
  int insert_id = mysql_insert_id(&borba_db);
  printf("Bomb with id %d was inserted\n", insert_id);

  return SUCCESS;

}

// FIX: do these transactions need to rollback if fail?
int db_begin_transaction()
{
  char query[255];

  sprintf(query, "BEGIN");

  if(mysql_real_query(&borba_db, query, strlen(query)) != 0){
    printf(mysql_error(&borba_db));
    return ERROR;
  }

  return SUCCESS;

}

int db_commit_transaction()
{
  char query[255];

  sprintf(query, "COMMIT");

  if(mysql_real_query(&borba_db, query, strlen(query)) != 0){
    printf(mysql_error(&borba_db));
    return ERROR;
  }

  return SUCCESS;
}

int db_rollback_transaction()
{
  char query[255];

  sprintf(query, "ROLLBACK");

  if(mysql_real_query(&borba_db, query, strlen(query)) != 0){
    printf(mysql_error(&borba_db));
    return ERROR;
  }

  return SUCCESS;
}

int db_lock_table(char *table)
{
  char query[255];

  sprintf(query, "LOCK TABLE %s", table);

  if(mysql_real_query(&borba_db, query, strlen(query)) != 0){
    printf(mysql_error(&borba_db));
    return -1;
  }

  return 0;
}

int db_unlock_all_tables()
{
  char query[255];

  sprintf(query, "UNLOCK TABLES");

  if(mysql_real_query(&borba_db, query, strlen(query)) != 0){
    printf(mysql_error(&borba_db));
    return -1;
  }

  return 0;
}

void map_str_to_array(const char *map_str, struct GameState *gs) {
  const char *p = map_str;
  for (int row = 0; row < NUM_ROWS; row++) {
    for (int col = 0; col < NUM_COLS; col++) {
      gs->map_info[row][col] = *p - '0';
      p++;
    }
  }
}

void map_array_to_str(char *map_str, const struct GameState *gs) {
  char *p = map_str;
  for (int row = 0; row < NUM_ROWS; row++) {
    for (int col = 0; col < NUM_COLS; col++) {
      *p = gs->map_info[row][col] + '0';
      p++;
    }
  }
  *p = '\0';
}


// OLD STUFF

/*
 // Check if the maximum number of bombs has already been planted
  sprintf(query,
	  "SELECT bombs_dropped FROM Player WHERE id = %d"
	  player_id);

  if(mysql_real_query(&borba_db, query, strlen(query))){
    rollback_transaction();
    printf(mysql_error(&borba_db));
    return -1;
  }

  res = mysql_store_result(&borba_db); // Download result from server
  row = mysql_fetch_row(res); // Get a row from the results

  int bombs_dropped = row[5];
  int bombs_max = row[6];

  printf("Player %d  has planted %d bombs (max: %d)\n", 
	 player_id, bombs_dropped, bombs_max);
  mysql_free_result(res); // Release memory used to store results.
 
  if(bombs_dropped >= bombs_max){
*/

/*
int update_table(int id, int value)
{
  char query[255];

  sprintf(query, "UPDATE borba_table SET value = %d WHERE id = %d", value, id);
  printf("query: %s\n", query);

  if(mysql_real_query(&borba_db, query, strlen(query))){
    printf(mysql_error(&borba_db));
    return -1;
  }

  return 0;
}

int borba_insert(int id, int value)
{
char query[255];
int insert_id;
MYSQL_RES *res;
MYSQL_ROW row;

sprintf(query, "INSERT INTO borba_table(id, value) VALUES ('%d', '%d')", 
id, value);
printf("query: %s\n", query);

if(mysql_real_query(&borba_db, query, strlen(query))){
printf(mysql_error(&borba_db));
return -1;
}

// this function actually increments id automatically, but we need to set
// our database flag AUTO_INCREMENT (not set now)
insert_id=mysql_insert_id(&borba_db); // Find what id that data was given

query = malloc(255);
//sprintf(query, "SELECT demodata FROM demotable WHERE id='%d'", insert_id);
sprintf(query, "SELECT id, value FROM borba_table WHERE id='%d'", id);
if(mysql_real_query(&borba_db, query, strlen(query))){ // Make query
printf(mysql_error(&borba_db));
exit(1);
}
 free(query);

 res=mysql_store_result(&borba_db); // Download result from server
 row=mysql_fetch_row(res); // Get a row from the results
 printf("You inserted '%s', '%s'.\n", row[0], row[1]);
 mysql_free_result(res); // Release memory used to store results.

 return 0;
}
*/ 
