// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

// CORBA includes
#include <OB/CORBA.h>
#include <Game.h>

// standard includes
#include <stdlib.h>

// project includes
#include "client.h"


#ifdef HAVE_STD_IOSTREAM
using namespace std;
#endif

int run(CORBA::ORB_ptr orb, int argc, char* argv[]) {
  //
  // Get "Game" object
  //
  CORBA::Object_var obj;

  try {
    obj  = orb -> string_to_object("relfile:/Game.ref");
  } catch(const CORBA::BAD_PARAM&) {
  }

  if (CORBA::is_nil(obj)) {
    cerr << argv[0] << ": cannot read IOR from Game.ref" << endl;
    return EXIT_FAILURE;
  }

  Game_var game = Game::_narrow(obj);
  assert(!CORBA::is_nil(game));

  // start the client code
  int rv = client_main(argc, argv, game);
  //game->shutdown();
  if (rv < 0) {
    cerr << "Unknown error occurred in client_main" << endl;
    return EXIT_FAILURE;
  }

  /*
  //
  // Main loop
  //
  cout << "Enter 'h' for hello, 's' for shutdown or 'x' for exit:\n";
  char c;
  int retval;
  do {
    cout << "> ";
    cin >> c;
    if (c == 'h') {
      struct ClientAction action;
      struct GameState gamestate;
      action.arrow = 31337;
      gamestate = game->update( action );
      printf( "Received [%d]\n", gamestate.errval );
    } else if (c == 's')
      game->shutdown();
  }
  while (cin.good() && c != 'x');
  */

  return EXIT_SUCCESS;
}

int main(int argc, char* argv[], char*[]) {
  int status = EXIT_SUCCESS;
  CORBA::ORB_var orb;

  try {
    orb = CORBA::ORB_init(argc, argv, "Game-Client");
    status = run(orb, argc, argv);
  } catch (const CORBA::Exception& ex) {
    cerr << ex << endl;
    status = EXIT_FAILURE;
  }

  if (!CORBA::is_nil(orb)) {
    try {
      orb -> destroy();
    } catch (const CORBA::Exception& ex) {
      cerr << ex << endl;
      status = EXIT_FAILURE;
    }
  }

  return status;
}
