/************
 * INCLUDES *
 ************/

// CORBA includes
#include <OB/CORBA.h>

// project includes
#include <Game_impl.h>
#include "sql_lib.h"
#include "game_constants.h"


#ifdef HAVE_STD_IOSTREAM
using namespace std;
#endif

// ----------------------------------------------------------------------
// Game_impl constructor
// ----------------------------------------------------------------------

Game_impl::Game_impl(CORBA::ORB_ptr orb, PortableServer::POA_ptr poa,
                     CORBA::ULong game_id)
    : orb_(CORBA::ORB::_duplicate(orb)), 
      poa_(PortableServer::POA::_duplicate(poa))
{
  // initialize the game state
  gs.game_id = game_id;
  this->init_round();

  // connect the database
  connect_to_database();

  // create the database table -- this returns an error if the table already
  // exists so be sure to fix this in the future
  create_tables();
}

// ----------------------------------------------------------------------
// Game_impl Destructor
// ----------------------------------------------------------------------

Game_impl::~Game_impl() {
  disconnect_database();
}

// ----------------------------------------------------------------------
// Game_impl private member implementation
// ----------------------------------------------------------------------

void Game_impl::init_round() {
  char map_str[NUM_ROWS * NUM_COLS + 1];

  // FIXME - set these values in the database

  // set the round number and remaining time
  gs.round_number = 1;
  gs.time_left    = 180;

  // initialize the map values
  strcpy(map_str, "002222222222200012121212121210222222222222222212121212121212222222222222222212121212121212222222222222222212121212121212222222222222222012121212121210002222222222200");
  this->map_str_to_array(map_str);

  // setup the players
  for (int i = 0; i < NUM_PLAYERS; i++) {
    gs.players[i].player_id = i;          // FIXME -- use real player_id here
    gs.players[i].status = PSTAT_ALIVE;
    gs.players[i].bombs_dropped = 0;
    gs.players[i].rounds_won = 0;
  }
  gs.players[0].x_coord = 0;
  gs.players[0].y_coord = 0;
  gs.players[1].x_coord = NUM_COLS - 1;
  gs.players[1].y_coord = 0;
  gs.players[2].x_coord = 0;
  gs.players[2].y_coord = NUM_ROWS - 1;
  gs.players[3].x_coord = NUM_COLS - 1;
  gs.players[3].y_coord = NUM_ROWS - 1;
}

void Game_impl::map_str_to_array(char *map_str) {
  char *p = map_str;
  for (int row = 0; row < NUM_ROWS; row++) {
    for (int col = 0; col < NUM_COLS; col++) {
      gs.map_info[row][col] = *p - '0';
      p++;
    }
  }
}

void Game_impl::map_array_to_str(char *map_str) {
  char *p = map_str;
  for (int row = 0; row < NUM_ROWS; row++) {
    for (int col = 0; col < NUM_COLS; col++) {
      *p = gs.map_info[row][col] + '0';
      p++;
    }
  }
  *p = '\0';
}

struct PlayerInfo* Game_impl::get_player(unsigned int pid) {
  for (int i = 0; i < NUM_PLAYERS; i++) {
    if (gs.players[i].player_id == pid) {
      return &gs.players[i];
    }
  }
  return NULL;
}


// ----------------------------------------------------------------------
// Game_impl public member implementation
// ----------------------------------------------------------------------

struct GameState
Game_impl::update(const struct ClientAction &c_action)
    throw(CORBA::SystemException)
{
  static unsigned long seq = 0;
  char map_str[NUM_ROWS * NUM_COLS + 1];
  struct PlayerInfo *pi = get_player(c_action.player_id);
  if (pi == NULL) {
    cerr << "ERROR: INVALID PLAYER ID";
    return gs;
  }
  unsigned int x   = pi->x_coord;
  unsigned int y   = pi->y_coord;
  unsigned int pid = pi->player_id;

  // fill in the GameState structure to send back to the client
  gs.seq_num = seq++;

  // setup the map
  // FIXME - get the map string from the database

  // plant a bomb if necessary
  // FIXME - get/set bomb info in the database
  if (c_action.request_bomb) {
    if (pi->bombs_dropped < MAX_BOMBS) {
      pi->bombs[pi->bombs_dropped].player_id = pid;
      pi->bombs[pi->bombs_dropped].x_coord = x;
      pi->bombs[pi->bombs_dropped].y_coord = y;
      pi->bombs_dropped++;
    }
  }

  // handle the player movement
  // FIXME - write these values to the database
  switch (c_action.arrow) {
    case MOVE_LEFT:
      if ((x > 0) && (gs.map_info[y][x-1] == MAP_EMPTY)) {
        pi->x_coord = pi->x_coord - 1;
      }
      break;
    case MOVE_RIGHT:
      if ((x < NUM_COLS-1) && (gs.map_info[y][x+1] == MAP_EMPTY)) {
        pi->x_coord = pi->x_coord + 1;
      }
      break;
    case MOVE_UP:
      if ((y > 0) && (gs.map_info[y-1][x] == MAP_EMPTY)) {
        pi->y_coord = pi->y_coord - 1;
      }
      break;
    case MOVE_DOWN:
      if ((y < NUM_ROWS-1) && (gs.map_info[y+1][x] == MAP_EMPTY)) {
        pi->y_coord = pi->y_coord + 1;
      }
      break;
  }

  // no error
  gs.err_val = 0;
  return gs;
}

long Game_impl::exit_game(CORBA::ULong player_id, CORBA::ULong game_id)
  throw(CORBA::SystemException)
{
  //Do Stuff
}

bool Game_impl::is_game_started(CORBA::ULong player_id, CORBA::ULong game_id)
  throw(CORBA::SystemException)
{
  //Do Stuff
}

PortableServer::POA_ptr
Game_impl::_default_POA()
{
    return PortableServer::POA::_duplicate(poa_);
}

long Game_impl::get_id()
  throw(CORBA::SystemException)
{
  return gs.game_id;
}
