#include <OB/CORBA.h>
#include <OB/BootManager.h>

#include <Game_impl.h>
#include <GameServer_impl.h>

#include <stdlib.h>
#include <errno.h>

#ifdef HAVE_FSTREAM
#   include <fstream>
#else
#   include <fstream.h>
#endif

#ifdef HAVE_STD_IOSTREAM
using namespace std;
#endif

#include <OB/CosNaming.h>

int run(CORBA::ORB_ptr orb, int argc, char* argv[]) {
  // Resolve Root POA
  CORBA::Object_var poaObj = orb->resolve_initial_references("RootPOA");
  PortableServer::POA_var rootPOA = PortableServer::POA::_narrow(poaObj);

  // Get a reference to the POA manager
  PortableServer::POAManager_var manager = rootPOA->the_POAManager();

  //Create implementation GameServer object
  GameServer_impl* gameServerImpl = new GameServer_impl(orb, rootPOA);
  PortableServer::ServantBase_var gameServer_servant = gameServerImpl;
  GameServer_var gameServer = gameServerImpl->_this();

  /*
  // Create implementation object
  // FIXME -- change the game_id constructor argument (last argument)
  Game_impl* gameImpl = new Game_impl(orb, rootPOA, 1337);
  PortableServer::ServantBase_var servant = gameImpl;
  Game_var game = gameImpl->_this();
  */

  // Get naming service
  CORBA::Object_var inc;

  try {
    inc = orb->resolve_initial_references("NameService");
  } catch (const CORBA::ORB::InvalidName&) {
    cerr << argv[0] << ":can't resolve `NameService'" << endl;
    return EXIT_FAILURE;
  }
  
  if (CORBA::is_nil(inc.in())) {
    cerr << argv[0] << ": `NameService' is a nil object reference" << endl;
    return EXIT_FAILURE;
  }
  
  CosNaming::NamingContext_var nc = CosNaming::NamingContext::_narrow(inc.in());

  if (CORBA::is_nil(nc.in())) {
    cerr << argv[0] << ": `NameService is not a NamingContext object reference"          << endl;
    return EXIT_FAILURE;
  }

  try {
    // Bind name with the Naming Service
    CosNaming::Name gameServerName;
    gameServerName.length(1);
    gameServerName[0].id = CORBA::string_dup("GameServer");
    nc->bind(gameServerName,gameServer.in());

    manager->activate();
    orb->run();

    // unbind object from naming service
    nc->unbind(gameServerName);
  } catch (const CosNaming::NamingContext::NotFound& ex) {
    cerr << argv[0] << ": Got a `NotFound' exception (";
    switch (ex.why) {
      case CosNaming::NamingContext::missing_node:
        cerr << "missing node";
        break;
      case CosNaming::NamingContext::not_context:
        cerr << "not context";
        break;
      case CosNaming::NamingContext::not_object:
        cerr << "not object";
        break;
    }
    cerr << ")" << endl;
    return EXIT_FAILURE;
  } catch (const CosNaming::NamingContext::CannotProceed&) {
    cerr << argv[0] << ": Got a `CannotProceed' exception" << endl;
    return EXIT_FAILURE;
  } catch (const CosNaming::NamingContext::InvalidName&) {
    cerr << argv[0] << ": Got an `InvalidName' exception" << endl;
    return EXIT_FAILURE;
  } catch (const CosNaming::NamingContext::AlreadyBound&) {
    cerr << argv[0] << ": Got an `AlreadyBound' exception" << endl;
    return EXIT_FAILURE;
  } catch (const CosNaming::NamingContext::NotEmpty&) {
    cerr << argv[0] << ": Got a `NotEmpty' exception" << endl;
    return EXIT_FAILURE;
  }
  return EXIT_SUCCESS;
}

int main(int argc, char* argv[], char*[]) {
  int status = EXIT_SUCCESS;
  CORBA::ORB_var orb;

  try {
    orb = CORBA::ORB_init(argc, argv);
    status = run(orb, argc, argv);
  } catch(const CORBA::Exception& ex) {
    cerr << ex << endl;
    status = EXIT_FAILURE;
  }

  if (!CORBA::is_nil(orb)) {
    try {
      orb -> destroy();
    } catch(const CORBA::Exception& ex) {
      cerr << ex << endl;
      status = EXIT_FAILURE;
    }
  }

  return status;
}

